import argparse
import ConfigParser
import os
import sys


def make_dir_if_not_exists(path):
    if not os.path.exists(path): os.makedirs(path)

home_dir = os.path.expanduser('~/.zot/')
make_dir_if_not_exists(home_dir)
config_file_path = os.path.join(home_dir, 'settings')


config = ConfigParser.SafeConfigParser()
if not config.has_section('zot'): config.add_section('zot')
config.read(config_file_path)

def get_zotero_dir():
    try:
        return config.get('zot', 'zotero_dir')
    except ConfigParser.NoOptionError:
        print '''No Zotero directory defined. Run 'zot path [zotero_dir]' to set your Zotero directory.'''
        sys.exit()
        
def write_zotero_dir(path):
    config.set('zot', 'zotero_dir', path)
    with open(config_file_path, 'w') as output_file:
        config.write(output_file)



import os
import sys
import sqlalchemy as sql
from item import Item


class Zotero:
    def __init__(self, zotero_dir):
        self.zotero_dir = zotero_dir
        self.zotero_db_path = os.path.abspath(os.path.join(zotero_dir, 'zotero.sqlite'))
        self.zotero_storage_path = os.path.join(zotero_dir, 'storage/')
        if not os.path.exists(self.zotero_db_path):
            raise Exception('Zotero db not found at %s.' % self.zotero_db_path)
        self.db = sql.create_engine('sqlite:///' + self.zotero_db_path)
        self.db.echo = False
        self.metadata = sql.MetaData(self.db)

        # tables
        self.items = sql.Table('items', self.metadata, autoload=True)
        self.fields = sql.Table('fields', self.metadata, autoload=True)
        self.item_data = sql.Table('itemData', self.metadata, autoload=True)
        self.item_data_values = sql.Table('itemDataValues', self.metadata, autoload=True)
        self.item_attachments = sql.Table('itemAttachments', self.metadata, autoload=True)
        self.collections = sql.Table('collections', self.metadata, autoload=True)
        self.collection_items = sql.Table('collectionItems', self.metadata, autoload=True)
        self.item_creators = sql.Table('itemCreators', self.metadata, autoload=True)
        self.creators = sql.Table('creators', self.metadata, autoload=True)
        self.creator_data = sql.Table('creatorData', self.metadata, autoload=True)
        self.item_notes = sql.Table('itemNotes', self.metadata, autoload=True)
        
        self.get_items()
        

    def get_items(self):
        # get all items and associated field names, and store in items dictionary
        query = sql.select([self.items.c.key, self.fields.c.fieldName, self.item_data_values.c.value], 
                           (self.items.c.itemID == self.item_data.c.itemID) &
                           (self.item_data.c.fieldID == self.fields.c.fieldID) &
                           (self.item_data.c.valueID == self.item_data_values.c.valueID)
                           )
        result = query.execute()
        items = {}
        for key, field_name, value in result:
            if not key in items: items[key] = {'key': key}
            if field_name == 'date':
                items[key]['year'] = value[:4]
            else: items[key][field_name] = value
            
        # get authors for these items
        query = sql.select([self.items.c.key, self.creator_data.c.lastName, self.creator_data.c.firstName],
                           (self.items.c.itemID == self.item_creators.c.itemID) &
                           (self.creators.c.creatorID == self.item_creators.c.creatorID) &
                           (self.creators.c.creatorDataID == self.creator_data.c.creatorDataID)
                           )
        result = query.execute()
        for key, last, first in result:
            if not key in items: items[key] = {'key': key}
            if not 'authors' in items[key]: items[key]['authors'] = []
            items[key]['authors'].append((last, first))
            
        # get all PDF attachments for these items
        query = sql.select([self.items.c.key, self.item_attachments.c.path],
                           (self.items.c.itemID == self.item_attachments.c.itemID) &
                           (self.item_attachments.c.mimeType == 'application/pdf')
                           )
        result = query.execute()
        for key, path in result:
            if not key in items: items[key] = {'key': key}
            if not 'attachments' in items[key]: items[key]['attachments'] = []
            items[key]['attachments'].append(path)
            
        # get all notes for these items
        query = sql.select([self.items.c.key, self.item_notes.c.note],
                           (self.items.c.itemID == self.item_notes.c.itemID)
                           )
        result = query.execute()
        for key, note in result:
            if not key in items: items[key] = {'key': key}
            if not 'notes' in items[key]: items[key]['notes'] = []
            items[key]['notes'].append(note)

        # get all collections
        query = sql.select([self.collections.c.collectionName, self.items.c.key],
                           (self.collections.c.collectionID == self.collection_items.c.collectionID) &
                           (self.collection_items.c.itemID == self.items.c.itemID)
                           )
        result = query.execute()
        collections = {}
        for collection, key in result:
            if not collection in collections: collections[collection] = set()
            collections[collection].add(key)
            if not key in items: items[key] = {'key': key}
            if not 'collections' in items[key]: items[key]['collections'] = set()
            items[key]['collections'].add(collection)
            
        self.all_items = {k: Item(v) for k, v in items.items()}

        
    def search(self, best=False, **kwargs):
        matches = [(item, item.match(**kwargs)) for item in self.all_items.values()]
        
        if not matches: return []
        if best: return [sorted(matches, key = lambda m: m[1], reverse=True)[0][0]]
        return [m[0] for m in filter(lambda m: m[1] > 0, matches)]
        
    def read(self, keys):
        pass
        
    def notes(self, keys):
        pass
        
    def add_note(self, key, note_txt):
        pass
        
    def bib(self, keys):
        pass



def help_msg():
    print '''Usage: zot (command) (args)
    
Commands:
    search [keywords]: return keys of all matching items
    best [keywords]: return key of best search match
    bib [keys]: view bibliography for one or more items
    cit [keys]: view citations for one or more items
    files [keys]: view all attached files for one or more items
    read [keys]: view text content of attached PDFs for one or more items
    notes [keys]: view all notes for one or more items
    path (zotero_dir): set the path to your Zotero directory
    help'''
        
def main():
    if len(sys.argv) < 2:
        help_msg()
        return
        
    command = sys.argv[1].lower()
    if command == 'help':
        help_msg()
        return
    elif command == 'path':
        arg = sys.argv[2]
        from settings import write_zotero_dir
        write_zotero_dir(arg)
        sys.exit()
        
    from settings import get_zotero_dir
    zotero_dir = get_zotero_dir()
    z = Zotero(zotero_dir)
    
    if len(sys.argv) > 2: args = sys.argv[2:]
    else:
        args = []
        for line in sys.__stdin__:
            args.append(line.strip())
    
    
    if command in ('search', 'best'):
        search_args = {}
        n = 0
        set_arg = 'keywords'
        while n < len(args):
            if not set_arg in search_args: search_args[set_arg] = []
            arg = args[n]
            if arg.startswith('--'): set_arg = arg[2:]
            else: search_args[set_arg].append(arg)
            n += 1
            
        result = z.search(best=command=='best', **search_args)
        for i in result:
            print getattr(i, 'key')
    elif command in ('bib', 'bibliography'):
        for result in [z.all_items[key].bibliography() for key in args]:
            print result
    elif command in ('cit', 'cite', 'citation'):
        for result in [z.all_items[key].citation() for key in args]:
            print result
    elif command == 'files':
        for result in [z.all_items[key] for key in args]:
            if hasattr(result, 'attachments'):
                for attachment in result.attachments:
                    print result.format_filename(attachment, z.zotero_storage_path)
    elif command == 'read':
        for key in args:
            z.all_items[key].get_full_text(z.zotero_storage_path)
    elif command == 'notes':
        for result in [z.all_items[key] for key in args]:
            if hasattr(result, 'notes'):
                for note in result.notes: print note
    elif command == 'debug':
        for i in [item.__dict__ for item in z.all_items.values()]: print i
    else:
        help_msg()
        return


if __name__ == '__main__':
    main()



import os
import pdf2txt
import re

def format_author(author):
    return '%s, %s' % author


class Item:
    def __init__(self, item_dict={}):
        self.__dict__ = item_dict
        
    def __repr__(self):
        return repr(self.__dict__)
        
    def match(self, **kwargs):
        if not 'keywords' in kwargs: kwargs['keywords'] = []
        fields_to_search = {'title': 'title',
                            'pub': 'publicationTitle',
                            'author': 'author_str',
                            'year': 'year',
                            'collection': 'collections',
                            }
        result = []
        match_score = 0
        for field, to_search in fields_to_search.items():
            if not hasattr(self, to_search): continue
            
            if field in kwargs: search_terms = kwargs['keywords'] + kwargs[field]
            else: search_terms = kwargs['keywords']
            
            text_to_search = getattr(self, to_search)
            if isinstance(text_to_search, set): text_to_search = ','.join(text_to_search)
            text_to_search = text_to_search
            search_terms = [term for term in search_terms]
            for search_term in search_terms:
                match_score += len(re.findall(search_term, text_to_search))
        
        return match_score
        
        
    def author_string(self):
        if not hasattr(self, 'authors'): return ''
        authors = getattr(self, 'authors')
        if len(authors) == 1: return format_author(authors[0])
        elif len(authors) == 2: return ' and '.join([format_author(a) for a in authors])
        else:
            return ', '.join([format_author(authors[n]) if n < len(authors)-1 else 'and %s' % format_author(authors[n]) for n in range(len(authors))])
            
    author_str = property(author_string)
        
        
    def bibliography(self):
        bib = self.author_string()
        if hasattr(self, 'year'): bib += ' %s.' % self.year
        if hasattr(self, 'title'): bib += ' %s.' % self.title
        if hasattr(self, 'publicationTitle'): bib += ' %s' % self.publicationTitle

        v = []
        if hasattr(self, 'volume'): v += ['Vol. %s' % self.volume]
        if hasattr(self, 'issue'): v += ['Issue %s' % self.issue]
        if hasattr(self, 'pages'): v += ['Pages %s' % self.pages]       
        if v: bib += ' ' + (', '.join(v))
        
        if v or hasattr(self, 'publicationTitle'): bib += '.'
        
        if hasattr(self, 'doi'): bib += ' doi:%s' % self.doi
        
        return bib
        
        
    def citation(self):
        citation = ''
        if hasattr(self, 'authors'): 
            citation = (self.authors[0][0])
            if len(self.authors) > 1: citation += ' et al.'
        else: citation = self.title
        if hasattr(self, 'year'): citation += ' %s.' % self.year
        
        return citation
        
        
    def format_filename(self, name, storage_dir):
        return name.replace('storage:', os.path.join(storage_dir, self.key + '/'))
        
        
    def get_full_text(self, storage_dir):
        if hasattr(self, 'attachments'):
            for attachment in self.attachments:
                # TODO: read text from pdf
                pdf2txt.main(['pdf2txt', self.format_filename(attachment, storage_dir)])
        else: 
            return "No PDF attachments."



from setuptools import setup
from __init__ import VERSION


setup(name='zot',
      version=VERSION,
      description='zotero command-line interface',
      author='Ben Morris',
      author_email='ben@bendmorris.com',
      url='https://github.com/hourchallenge/zot',
      packages=['zot'],
      package_dir={
                'zot':''
                },
      entry_points={
        'console_scripts': [
            'zot = zot.zot:main',
        ],
      },
      install_requires=[
                        'sqlalchemy',
                        'pdfminer',
                        ]
      )
